import numpy as np
import torch
import torch.nn as nn
torch.set_default_dtype(torch.float32)

cnn_size=[(64,False),(128,True),(256,False),(512,False)]
lin_size=[(256,False,0),(128,False,0)]
cnn_kernel=2
mp_kernel=2

class Convfeat(nn.Module):
    def __init__(self, length):
        super(Convfeat, self).__init__()
        
        cuda0 = torch.device('cuda:0')
        self.conv = nn.ModuleList()
        cur_in_channel = 1
        for s,use_mp in cnn_size:
            c = torch.nn.Conv3d(in_channels=cur_in_channel, out_channels=s, kernel_size=(2,2,2)).to(cuda0)
            mp = torch.nn.MaxPool3d(kernel_size=2, stride=2).to(cuda0) if use_mp else None
            self.conv.append(c)
            self.conv.append(torch.nn.ReLU())
            if use_mp: self.conv.append(mp)
            cur_in_channel=s
            
        self.last=cur_in_channel

    def forward(self, x):
        for l in self.conv:
            x = l(x)
        x = torch.flatten(x, 1)
        return x


class CNNnet(nn.Module):
    def __init__(self, length=8):
        super(CNNnet, self).__init__()

        cuda0 = torch.device('cuda:0')
        self.convfeat = Convfeat(length)
        self.parts = nn.ModuleList()
        for s,use_mp in cnn_size:     
            length -= (cnn_kernel-1)
            if(use_mp): length //= mp_kernel
        last = length * cnn_size[-1][0] 

        for i,ub,dp in lin_size:
            self.parts.append(nn.Linear(last, i))
            self.parts.append(nn.BatchNorm1d(i)) if ub else None
            self.parts.append(nn.Tanh())
            self.parts.append(nn.Dropout(p=dp)) if dp>0 else None
            last = i
            
        self.parts = self.parts.to(cuda0)
        self.last_lin = nn.Linear(last, 1).to(cuda0)


    def forward(self, input):
        cuda0 = torch.device('cuda:0')  
        input = input.to(cuda0)
        x = self.convfeat(input)

        for l in self.parts:
            x = l(x)
             
        x = self.last_lin(x)
        
        return x
