#!/usr/bin/env python3

import argparse
import os
import random
import math
import torch
import numpy as np
import time
from dataset import DataSampler
from dataset import PatchDataset
from cnnnet import CNNnet
from pathlib import Path

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    
    parser.add_argument('--name', type=str, default='', help='trained neural network model name, should end with .pth')
    parser.add_argument('--width', type=int, default='8', help='feature width')
    opt=parser.parse_args()
    w = opt.width    
    cnnnet = CNNnet(length=w)

    assert opt.name, "model name needs to be specified using --name"
    if not opt.name.endswith(".pth"):
        opt.name += ".pth"
        print(f"Appending \".pth\" to model name ...")
    
    cnnnet.load_state_dict(torch.load("models/"+opt.name, weights_only=True))
    model_name = opt.name[:-4]        
    
    input_example = torch.rand(256,1,w,w,w)
    traced_script_module = torch.jit.trace(cnnnet,input_example)    
    traced_script_module.save(model_name+".pt")
    
        
