#!/bin/bash

DUMP_PT=1
DUMP_PHI=1
DUMP_BGEO=1
DUMP_VDB=1
#--------------------------------------------------------------------------------
# parameters
#--------------------------------------------------------------------------------
stride=2  #refined level
m0_radius=3 #radius of density and feature kernel
dx=0.005 #level set grid dx
#lb="-0.1 -0.1 -0.1" #minimum corner of level set domain
lbx=-0.1 # min corner x
lby=-0.1 # min corner y
lbz=-0.1 # min corner z
N="240 240 240" # grid resolution

# TODO: check params
function usage {
    if [ "$#" -ne 5 ]; then
        echo "$#"
        echo "Usage: $0 <model> <input data dir> <output phi dir> <start frame> <end frame>"
        exit 1
    fi
}

# command line parameters
usage "$@"
model=$1
particle_data_dir=$2
output_dir=$3
start_frame=$4
end_frame=$5


# convert .pth model to .pt
if [ $DUMP_PT == 1 ] ; then
    ./convert_to_pt.py --name $model
fi

# dump a grid array of phi values
if [ $DUMP_PHI == 1 ] ; then
    mkdir -p $output_dir
    phi_dir="$output_dir/phi"
    mkdir -p $phi_dir

    rm $phi_dir/* 2>/dev/null
    model_pt="${model%.pth}".pt

    for (( i=start_frame; i<=end_frame; i++ )); do
        printf -v line "X-%03d" "$((i))"
        echo $line
    done | xargs -P 10 -I % \
                 ./evaluate \
                 -dx $dx \
                 -lb "$lbx $lby $lbz" \
                 -N "$N" \
                 -i  $particle_data_dir/%.xyz \
                 -o  $phi_dir/%.phi \
                 -grid $particle_data_dir/X.grid \
                 -stride $stride \
                 -m0_r $m0_radius \
                 -model $model_pt 
fi

# dump bgeo file
if [ $DUMP_BGEO == 1 ]; then
    echo "Dumping xyz to bgeo"
    bgeo_dir="$output_dir/bgeo"
    mkdir -p $bgeo_dir
    rm $bgeo_dir/* 2>/dev/null

    for (( i=start_frame; i<=end_frame; i++ )); do
        echo $i
    done | xargs -P 40 -I @ \
                 ./xyz2bgeo \
                 -i_frame @ \
                 -i_pattern $particle_data_dir/X-%03d.xyz \
                 -o_pattern $bgeo_dir/%04d.bgeo
fi

    
# dump vdb file
# compute bx,by,and bz from lb and dx
if [ $DUMP_VDB == 1 ]; then
    echo "Dumping phi values to vdb"
    phi_dir="$output_dir/phi"
    vdb_dir="$output_dir/vdb"
    mkdir -p $vdb_dir
    rm $vdb_dir/* 2>/dev/null

    for (( i=start_frame; i<=end_frame; i++ )); do
        echo $i
    done | xargs -P 40 -I @ \
                 ./levelset2vdb -i_grid $particle_data_dir/X.grid \
                 -i_frame @ \
                 -i_pattern $phi_dir/X-%03d.phi.gz \
                 -o_pattern $vdb_dir/%04d.vdb \
                 -lbx $lbx -lby $lby -lbz $lbz 
fi
