#include <Core/Geometry_Particles/GEOMETRY_PARTICLES.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Read_Write/FILE_UTILITIES.h>
#include <Core/Utilities/VIEWER_DIR.h>
#include <Core/Viewer/VIEWER_OUTPUT.h>

using namespace PhysBAM;
using std::sin;
using std::cos;
typedef double T;
typedef VECTOR<T,3> TV;
typedef VECTOR<int,3> IV;

int main(int argc,char *argv[])
{
    T y_off=.01;
    T x_off=1;
    int frame=100;
    std::string output="output";
    
    PARSE_ARGS parse_args(argc,argv);
    parse_args.Add("-y",&y_off,"res","y separation");
    parse_args.Add("-x",&x_off,"dir","initial separation");
    parse_args.Add("-frame",&frame,"frames","number of frames to run.");
    parse_args.Add("-o",&output,"dir","output directory");
    parse_args.Parse();

    // // Set opengl viewer
    VIEWER_DIR viewer_dir(output);
    VIEWER_OUTPUT viewer_output(Stream_Type(0.),viewer_dir);
    viewer_output.template Add_Axes<TV::m>("axes",{});

    GEOMETRY_PARTICLES<TV> particles;
    particles.Store_Velocity();
    particles.Add_Elements(2);
    particles.V(0)={x_off,0,0};
    particles.V(1)={-x_off,0,0};

    viewer_output.Add("particles",particles,false,{.draw=true});
    for(int f=0;f<=frame;f++)
    {
        T t=2.0*f/frame;
        particles.X(0)={(1-x_off*(1-t))/2,(T).5-y_off/2,(T).5};
        particles.X(1)={(1+x_off*(1-t))/2,(T).5+y_off/2,(T).5};
        Flush_Frame(LOG::sprintf("%d",f).c_str());
        Write_To_File(Stream_Type(0.),LOG::sprintf("%s/X-%03d.xyz",output,f),particles.X);
    }
    return 0;
}
