#include <Core/Basic_Geometry/SPHERE.h>
#include <Core/Geometry_Particles/GEOMETRY_PARTICLES.h>
#include <Core/Meshes/SEGMENTED_CURVE.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Random_Numbers/RANDOM_NUMBERS.h>
#include <Core/Read_Write/FILE_UTILITIES.h>
#include <Core/Seeding/POISSON_DISK.h>
#include <Core/Utilities/VIEWER_DIR.h>
#include <Core/Viewer/VIEWER_OUTPUT.h>

using namespace PhysBAM;
using std::sin;
using std::cos;
typedef double T;
typedef VECTOR<T,3> TV;
typedef VECTOR<int,3> IV;

int main(int argc,char *argv[])
{
    T r=.5;
    std::string output="output";
    int seed=1234;
    T ppc=0.01;

    PARSE_ARGS parse_args(argc,argv);
    parse_args.Add("-r",&r,"radius","radius of object");
    parse_args.Add("-o",&output,"dir","output directory");
    parse_args.Add("-seed",&seed,"seed","random seed for particles");
    parse_args.Add("-ppc",&ppc,"ppc","Particles per cell");
    parse_args.Parse();

    // // Set opengl viewer
    VIEWER_DIR viewer_dir(output);
    VIEWER_OUTPUT viewer_output(Stream_Type(0.),viewer_dir);
    viewer_output.template Add_Axes<TV::m>("axes",{});

    RANDOM_NUMBERS<T> random;
    random.Set_Seed(seed);

    SPHERE<TV> sphere(TV()+0.5,r);
    
    ARRAY<TV> X0;
    POISSON_DISK<TV> pd(1);
    pd.Set_Distance_By_Volume(0.005*0.005*0.005/ppc);
    pd.Sample(random,sphere.Bounding_Box(),X0);

    int k=0;
    for(int i=0;i<X0.m;i++)
        if(sphere.Signed_Distance(X0(i))<0)
            X0(k++)=X0(i);
    X0.Resize(k);
    LOG::printf("particles: %P\n",X0.m);
    
    GEOMETRY_PARTICLES<TV> particles;    
    viewer_output.Add("particles",particles,false,{.draw=true});
    particles.Add_Elements(X0.m);
    particles.X=X0;

    Flush_Frame(LOG::sprintf("%d",0).c_str());
    Write_To_File(Stream_Type(0.),LOG::sprintf("%s/sphere.%f.xyz",output,ppc),particles.X);
    return 0;
}
