#include <Core/Geometry_Particles/GEOMETRY_PARTICLES.h>
#include <Core/Meshes/SEGMENTED_CURVE.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Random_Numbers/RANDOM_NUMBERS.h>
#include <Core/Read_Write/FILE_UTILITIES.h>
#include <Core/Seeding/POISSON_DISK.h>
#include <Core/Utilities/VIEWER_DIR.h>
#include <Core/Viewer/VIEWER_OUTPUT.h>

using namespace PhysBAM;
using std::sin;
using std::cos;
typedef double T;
typedef VECTOR<T,3> TV;
typedef VECTOR<int,3> IV;

T phi(const TV& X)
{
    T x=X.x,y=X.y,z=X.z;
    T a=-.2;
    return 60*y*(a*a+a+1)*(a*a+3*a+3)*z*z*z*(x*x-1./3*y*y)*(a+2)*a*sqrt(2)+(y*y*y*y*y*y+(39*x*x-21*z*z)*y*y*y*y+(-21*x*x*x*x-42*x*x*z*z+24*z*z*z*z)*y*y+5*x*x*x*x*x*x-21*x*x*x*x*z*z+24*x*x*z*z*z*z-5)*a*a*a*a*a*a+(6*y*y*y*y*y*y+(234*x*x-126*z*z)*y*y*y*y+(-126*x*x*x*x-252*x*x*z*z+144*z*z*z*z)*y*y+30*x*x*x*x*x*x-126*x*x*x*x*z*z+144*x*x*z*z*z*z-30)*a*a*a*a*a+(15*y*y*y*y*y*y+(585*x*x-315*z*z)*y*y*y*y+(-315*x*x*x*x-630*x*x*z*z+360*z*z*z*z)*y*y+75*x*x*x*x*x*x-315*x*x*x*x*z*z+360*x*x*z*z*z*z-75)*a*a*a*a+(20*y*y*y*y*y*y+(780*x*x-420*z*z)*y*y*y*y+(-420*x*x*x*x-840*x*x*z*z+480*z*z*z*z)*y*y+100*x*x*x*x*x*x-420*x*x*x*x*z*z+480*x*x*z*z*z*z-100)*a*a*a+(15*y*y*y*y*y*y+(585*x*x-315*z*z)*y*y*y*y+(-315*x*x*x*x-630*x*x*z*z+360*z*z*z*z)*y*y+75*x*x*x*x*x*x-315*x*x*x*x*z*z+360*x*x*z*z*z*z-75)*a*a+(6*y*y*y*y*y*y+(234*x*x-126*z*z)*y*y*y*y+(-126*x*x*x*x-252*x*x*z*z+144*z*z*z*z)*y*y+30*x*x*x*x*x*x-126*x*x*x*x*z*z+144*x*x*z*z*z*z-30)*a+5*(y*y*y*y+(2*x*x+2*z*z+1)*y*y+x*x*x*x+(2*z*z+1)*x*x+z*z*z*z+z*z+1)*(x*x+y*y+z*z-1);
}

int main(int argc,char *argv[])
{
    T r=.2;
    int frame=1;
    std::string output="output";
    int seed=1234;
    T h=.01;
    T omega=.1;
    T vel=1;
    T ppc=0.01;

    PARSE_ARGS parse_args(argc,argv);
    parse_args.Add("-r",&r,"radius","radius of object");
    parse_args.Add("-frame",&frame,"frames","number of frames to run.");
    parse_args.Add("-o",&output,"dir","output directory");
    parse_args.Add("-h",&h,"dist","particle separation scale for seeding");
    parse_args.Add("-seed",&seed,"seed","random seed for particles");
    parse_args.Add("-ppc",&ppc,"ppc","Particles per cell");
    parse_args.Parse();

    // // Set opengl viewer
    VIEWER_DIR viewer_dir(output);
    VIEWER_OUTPUT viewer_output(Stream_Type(0.),viewer_dir);
    viewer_output.template Add_Axes<TV::m>("axes",{});

    RANDOM_NUMBERS<T> random;
    random.Set_Seed(seed);
    

    for(int i=1;i<=5;i++)
    {        
        ppc*=2;
        ARRAY<TV> X0;
        POISSON_DISK<TV> pd(1);
        pd.Set_Distance_By_Volume(0.01*0.01*0.01/ppc);
        pd.Sample(random,RANGE<TV>::Centered_Box(),X0);
    
        int k=0;
        for(int i=0;i<X0.m;i++)
            if(phi(X0(i))<0)
                X0(k++)=X0(i);
        X0.Resize(k);
        LOG::printf("particles: %P\n",X0.m);
    
        GEOMETRY_PARTICLES<TV> particles;
        particles.Add_Elements(X0.m);
        if(i==1) viewer_output.Add("particles",particles,false,{.draw=true});

        T t=0;
        ROTATION<TV> R(omega*t,TV(1,1,0));
        for(int i=0;i<X0.m;i++)
            particles.X(i)=R*X0(i)*r+TV(r+vel*t,(T).5,(T).5);

        Flush_Frame(LOG::sprintf("%d",i).c_str());
        Write_To_File(Stream_Type(0.),LOG::sprintf("%s/X-%03d.xyz",output,i-1),particles.X);
    }
    return 0;
}
