#include <Core/Grids/GRID.h>
#include <Core/Level_Sets/LEVELSET.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Read_Write/FILE_UTILITIES.h>
#include <string>

#ifdef USE_OPENVDB
#include <openvdb/openvdb.h>
#endif

using namespace PhysBAM;
typedef double T;
typedef VECTOR<T,3> TV;
typedef VECTOR<int,3> IV;
typedef T RW;

int main(int argc,char *argv[])
{
#ifdef USE_OPENVDB    
    std::string file_grid,dir;
    std::string frame;
    std::string input_filename_pattern;
    std::string output_filename_pattern;
    T lbx(0),lby(0),lbz(0);

    PARSE_ARGS parse_args(argc,argv);
    parse_args.Add("-i_grid",&file_grid,"file","Input grid file");
    parse_args.Add("-i_frame",&frame,"frame","frame number");
    parse_args.Add("-i_pattern",&input_filename_pattern,"input","Input phi file name. Use format string (e.g. X-%03d)");
    parse_args.Add("-o_pattern",&output_filename_pattern,"output","Output vdb file name. Use format string (e.g. %04d)");
    parse_args.Add("-lbx",&lbx,"lbx","lower bound of x, default 0");
    parse_args.Add("-lby",&lby,"lby","lower bound of y, default 0");
    parse_args.Add("-lbz",&lbz,"lbz","lower bound of z, default 0");
    parse_args.Parse();
    
    PHYSBAM_ASSERT(file_grid.length()>0);
    PHYSBAM_ASSERT(frame.length()>0);
    PHYSBAM_ASSERT(input_filename_pattern.length()>0);
    PHYSBAM_ASSERT(output_filename_pattern.length()>0);
    
    GRID<TV> grid;
    Read_From_File(file_grid,grid);

    ARRAY<T,IV> node_phi;
    Read_From_File(LOG::sprintf(input_filename_pattern,frame),node_phi);

    T voxel_w=grid.dX.Min();
    TV mc=grid.Domain().min_corner;
        
    openvdb::initialize();
    openvdb::FloatGrid::Ptr grid_vdb = openvdb::createLevelSet<openvdb::FloatGrid>(/*voxel size=*/voxel_w,/*half_width=*/4);
    grid_vdb->setName("LevelSet");

    openvdb::math::Transform::Ptr transform=grid_vdb->transformPtr();
    transform->postTranslate(openvdb::Vec3d(mc.x,mc.y,mc.z));
    
    openvdb::FloatGrid::Accessor accessor=grid_vdb->getAccessor();    
    for(RANGE_ITERATOR<TV::m> it(grid.Node_Indices());it.Valid();it.Next())
    {
        openvdb::Coord ijk(it.index.x,it.index.y,it.index.z);        
        T phi=node_phi(it.index);        
        accessor.setValue(ijk,phi);
    }
    
    openvdb::io::File(LOG::sprintf(output_filename_pattern,frame)).write({grid_vdb});
#endif    
    return 0;
    
}
