#!/bin/bash

GEN_TESTS=1
EVAL_TESTS=1
#--------------------------------------------------------------------------------
# Available Tests
# 1. two-particles
# 2. line-particles
# 3. plane-particles
# 4. translate-rotate
# 5. varied-ppc
# 6. vortex_deforming
#--------------------------------------------------------------------------------

modelname=$1
TEST_DIR="fixed-tests"
TEST_LISTS="1 2 3 4 5 6"

for i in $TEST_LISTS; do    
    DATA_DIR=test-"$i"-data

    # generate tests
    if [ $GEN_TESTS == 1 ] ; then
        cd $TEST_DIR    
        rm -rf $DATA_DIR
        mkdir $DATA_DIR
        case $i in
            1)
                echo "Generating test 1 (two particles)"
                ./run-two-particles.sh $DATA_DIR
                ;;
            2)
                echo "Generating test 2 (line particles)"
                ./run-line-particles.sh $DATA_DIR
                ;;
            3)
                echo "Generating test 3 (plane particles)"
                ./run-plane-particles.sh $DATA_DIR
                ;;
            4)
                echo "Generating test 4 (translate rotate)"
                ./run-translate-rotate.sh $DATA_DIR
                ;;
            5)
                echo "Generating test 5 (varied ppc)"
                ./run-varied-ppc.sh $DATA_DIR
                ;;
            6)
                echo "Generating test 6 (vortex deformation)"
                ./run-vortex-deformation.sh $DATA_DIR
                ;;
            *)
                echo "Invalid test number: $i"
                ;;
        esac
        cd ..
    fi

    # evaluate tests
    if [ $EVAL_TESTS == 1 ] ; then        
        echo "Running test $i"    
        frames=$(find $TEST_DIR/$DATA_DIR -type f -name "*.xyz.gz" | wc -l)
        ./eval_model.sh $modelname $TEST_DIR/test-"$i"-data $TEST_DIR/test-"$i"-out 0 $((frames - 1))
    fi

done
