import argparse
import os
import torch
import numpy as np
from pathlib import Path

def parse_arguments():
    parser = argparse.ArgumentParser()
    dir_home = Path.home()

    # naming / file handling
    parser.add_argument('--indir', type=str, default=str(dir_home)+'/PhysBAM/Projects/surface_reconstruction_nn/cnn_dataset',
                        help='input folder (point clouds)')
    parser.add_argument('--inputs', type=str, nargs='+', default=['.m0'], help='inputs of the network')
    parser.add_argument('--outputs', type=str, nargs='+', default=['.depth'], help='outputs of the network, a list with elements of:\n'
                        'depth: levelset scalar\n')
    parser.add_argument('--trainset', type=str, nargs='+',
                        default=["training.0"], help='training set file name')
    parser.add_argument('--testset', type=str, nargs='+',
                        default=["validation.0"], help='test set file name')
    parser.add_argument('--width', type=str, default='10', help='feature width')
    parser.add_argument('--shape_names',type=str, nargs='+', default=["training.0"],help='name of shape')
    return parser.parse_args()

def save_as_numpy(opt):
    shape_names=opt.shape_names
    ext_list = opt.inputs + opt.outputs
    w = int(opt.width)
    # K = 100
    for s in shape_names:
        print('save numpy format for shape %s' % (s))
        for i in ext_list:
            feat_filename = os.path.join(opt.indir, s + i)
            feat = np.loadtxt(feat_filename).astype('float32')
            if i == '.depth':
                feat = feat[...,np.newaxis]
                # feat = torch.tensor(np.reshape(feat, (feat.shape[0],w,w,w,feat.shape[1]//(w*w*w))))   
                # feat = feat.permute(0,4,1,2,3).to(torch.float32)
                    
            if (('grad' in i) and ('depth' in i)):
                feat = feat
                print(feat.shape)
                
            if i in opt.inputs:
                feat = torch.tensor(np.reshape(feat, (feat.shape[0],w,w,w,feat.shape[1]//(w*w*w))))               
                feat = feat.permute(0,4,1,2,3).to(torch.float32)
                # feat = torch.tensor(np.reshape(feat, (feat.shape[0],K,3)))
                if (('grad' in i) and ('cent' in i)): feat = np.reshape(feat, (feat.shape[0],3,3,w,w,w))
                elif(('grad' in i) and ('vol' in i)): feat = feat.unsqueeze(2)
                elif(('grad' in i) and ('m0' in i)): feat = feat.unsqueeze(2)

            np.save(feat_filename+'.npy', feat)


if __name__ == '__main__':
    opt = parse_arguments()
    save_as_numpy(opt)

