#include <Core/Arrays/ARRAYS_ND.h>
#include <Core/Grids/GRID.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Random_Numbers/RANDOM_NUMBERS.h>
#include <Core/Read_Write/FILE_UTILITIES.h>
#include <fstream>
#include <map>
#include <string>

using namespace PhysBAM;
typedef double T;
typedef VECTOR<T,3> TV;
typedef VECTOR<int,3> IV;
typedef T RW;

int main(int argc,char *argv[])
{
    ARRAY<std::string> i_prefix,i_shape;
    ARRAY<int> i_num_train,i_num_valid;
    std::string o_train_prefix,o_valid_prefix;
    std::string t_names,v_names;
    int seed=123;
    ARRAY<std::string> ext_list;

    PARSE_ARGS parse_args(argc,argv);
    parse_args.Add("-seed",&seed,"seed","Random number seed");
    parse_args.Add("-e",&ext_list,"list","List of file suffixes");
    parse_args.Add("-t_prefix",&o_train_prefix,"file","Output training prefix filename");
    parse_args.Add("-v_prefix",&o_valid_prefix,"file","Output validation prefix filename");
    parse_args.Add("-t_names",&t_names,"file","File with training data filenames");
    parse_args.Add("-v_names",&v_names,"file","File with validation data filenames");
    parse_args.Add("-p",&i_prefix,"pattern","printf pattern for conv files"); 
    parse_args.Add("-t",&i_num_train,"num","number of training samples to collect");
    parse_args.Add("-v",&i_num_valid,"num","number of validation samples to collect");
    parse_args.Add("-shape",&i_shape,"shape","name of shapes currently selected");
    parse_args.Parse();

    RANDOM_NUMBERS<T> rand;
    rand.Set_Seed(seed);

    std::ofstream t_namestream(t_names);
    std::ofstream v_namestream(v_names);
    ARRAY<std::ofstream> t(ext_list.m),v(ext_list.m);
    int nt=0,nv=0;
    int fsize=100000;

    PHYSBAM_ASSERT(i_prefix.m==i_shape.m);
    PHYSBAM_ASSERT(i_prefix.m==i_num_train.m);
    PHYSBAM_ASSERT(i_prefix.m==i_num_valid.m);
    for(int i=0;i<i_prefix.m;i++)
    {
        std::multimap<int,ARRAY<std::string> > keep;
        ARRAY<ARRAY<std::string>> keep_train,keep_valid;
        int num_keep=i_num_train(i)+i_num_valid(i);
        
        for(int j=0;;j++)
        {    
            LOG::printf("prefix %P %i\n",i_prefix(i),j);

            ARRAY<std::ifstream> in(ext_list.m);
            bool ok=true;
            for(int k=0;k<ext_list.m;k++)
            {
                in(k).open(LOG::sprintf(i_prefix(i),j)+ext_list(k));
                if(!in(k))
                {
                    ok=false;
                    break;
                }
            }
            if(!ok) break;
            std::ifstream in_gc,in_gd;
            std::string line_c,line_d,line_gc,line_gd;
            ARRAY<std::string> line(ext_list.m);
            while(1)
            {
                bool ok=true;
                for(int k=0;k<ext_list.m;k++)
                    if(!getline(in(k),line(k)))
                    {
                        ok=false;
                        break;
                    }
                if(!ok) break;

                

                int id=rand.Get_Uniform_Integer(0,INT_MAX);

                keep.insert({id,line});
                if(keep.size()>num_keep)
                    keep.erase(keep.begin());
            }
        }

        LOG::printf("%P: found %P, need %P\n",i_shape(i),keep.size(),num_keep);
        PHYSBAM_ASSERT(keep.size()==num_keep);

        auto it=keep.begin();
        for(int j=0;j<i_num_train(i);j++)
            keep_train.Append((it++)->second);
        for(int j=0;j<i_num_valid(i);j++)
            keep_valid.Append((it++)->second);
        
        for(const auto& p:keep_train)
        {
            if(nt%fsize==0)
            {
                for(int j=0;j<ext_list.m;j++){
                    if(t(j).is_open()) t(j).close();
                    t(j).open(o_train_prefix+"."+std::to_string(nt/fsize)+ext_list(j));
                }
                
                t_namestream<<"training."+std::to_string(nt/fsize)<<std::endl;
            }
        
            for(int j=0;j<ext_list.m;j++)
                t(j)<<p(j)<<std::endl;
            nt++;
        }

        for(const auto& p:keep_valid)
        {
            if(nv%fsize==0)
            {
                for(int j=0;j<ext_list.m;j++){
                    if(v(j).is_open()) v(j).close();
                    v(j).open(o_valid_prefix+"."+std::to_string(nv/fsize)+ext_list(j));
                }
                v_namestream<<"validation."+std::to_string(nv/fsize)<<std::endl;
            }         
                
            for(int j=0;j<ext_list.m;j++)
                v(j)<<p(j)<<std::endl;
            nv++;
        }
        
    }
    
    return 0;
}
