#include <Core/Arrays/ARRAY.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Read_Write/FILE_UTILITIES.h>

using namespace PhysBAM;
typedef float T;
typedef VECTOR<T,3> TV;

#ifdef USE_PARTIO
#include <Partio.h>

void writeXYZToBgeo(const ARRAY<TV>& X,const std::string& bgeoFilename) {    

    // Create a Partio particle set
    Partio::ParticlesDataMutable* particles=Partio::create();
    if (!particles) {
        PHYSBAM_FATAL_ERROR("Error: Unable to create Partio particle set.");
        return;
    }

    Partio::ParticleAttribute positionAttr=particles->addAttribute("position",Partio::VECTOR,3);
    for(int i=0;i<X.m;i++)
    {
        int particle=particles->addParticle();
        T* pos=particles->dataWrite<T>(positionAttr,particle);
        pos[0]=X(i).x;
        pos[1]=X(i).y;
        pos[2]=X(i).z;        
    }

    try{
        Partio::write(bgeoFilename.c_str(),*particles);
    }catch(const std::exception& e) {
        PHYSBAM_FATAL_ERROR("Error: Unable to write to bgeo file.");}

    particles->release();
}

int main(int argc,char* argv[]) {
    std::string file_xyz_pattern,file_bgeo_pattern;
    std::string frame;
    
    PARSE_ARGS parse_args(argc,argv);
    parse_args.Add("-i_frame",&frame,"frame","frame number");
    parse_args.Add("-i_pattern",&file_xyz_pattern,"input","Input phi file name. Use format string (e.g. X-%03d)");
    parse_args.Add("-o_pattern",&file_bgeo_pattern,"output","Output vdb file name. Use format string (e.g. %04d)");
    parse_args.Parse();

    PHYSBAM_ASSERT(frame.length()>0);
    PHYSBAM_ASSERT(file_xyz_pattern.length()>0);
    PHYSBAM_ASSERT(file_bgeo_pattern.length()>0);

    std::string file_xyz=LOG::sprintf(file_xyz_pattern,frame);
    std::string file_bgeo=LOG::sprintf(file_bgeo_pattern,frame);
    
    ARRAY<TV> X;
    Read_From_File(file_xyz,X);
    writeXYZToBgeo(X,file_bgeo);

    return 0;
}
#else
int main(int argc,char* argv[])
{
    PHYSBAM_FATAL_ERROR("You must enable USE_PARTIO to use this.");
    return 0;
}
#endif
